# Generated by Django 4.2.11 on 2024-07-13 15:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("ledger", "0002_delete_corpsteuer_alter_characteraudit_options_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="characteraudit",
            options={
                "default_permissions": (),
                "permissions": (
                    ("char_audit_manager", "Has access to all characters for own Corp"),
                    ("char_audit_admin_manager", "Has access to all characters"),
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="corporationaudit",
            options={
                "default_permissions": (),
                "permissions": (
                    ("corp_audit_manager", "Has access to own Corporation"),
                    ("corp_audit_admin_manager", "Has access to all Corporations"),
                ),
            },
        ),
    ]
