{% load i18n %}
{% load navactive %}

{% if perms.ledger.basic_access %}
<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:corporation_ledger' corporation_pk=0 %}">Corporation Ledger</a>
</li>

<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:character_ledger' character_pk=0 %}">Character Ledger</a>
</li>

<li class="nav-item">
    <span class="nav-link">|</span>
</li>
<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:ledger_add_char' %}">Add Char</a>
</li>
{% endif %}

{% if perms.ledger.admin_access or request.user.is_superuser %}
<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:ledger_fetch_memberaudit' %}">Add All Memberaudit Chars</a>
</li>
{% endif %}

<li class="nav-item">
    <span class="nav-link">|</span>
</li>
<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:character_admin' %}">Character Overview</a>
</li>
