{% load i18n %}
{% load humanize %}
{% load static %}

<div role="tabpanel" class="tab-pane active fade show" id="tab-current_month">
    <div class="card-body bg-secondary table-responsive">
        <div id="loadingIndicator-Month" class="d-flex justify-content-center align-items-center">
            {% if NIGHT_MODE %}
            <img src="{% static 'killstats/img/Spinner-1s-64px-dark.gif' %}">
            {% else %}
                <img src="{% static 'killstats/img/Spinner-1s-64px-light.gif' %}">
            {% endif %}
        </div>
        <div id="errorHandler" class="d-flex d-none justify-content-center align-items-center">
            You don't have Permission to view this Page
        </div>
        {% include 'ledger/corpledger/partials/table.html' with mode="Month" %}
    </div>
    <br>
    <div class="rounded-top">
        <div class="row g-0">
            <div class="col-md-6 g-2 d-none" id="ChartContainer-Month">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Ratting Chart</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="chart-loading-Month" class="d-flex justify-content-center align-items-center">
                        {% if NIGHT_MODE %}
                        <img src="{% static 'killstats/img/Spinner-1s-64px-dark.gif' %}">
                        {% else %}
                            <img src="{% static 'killstats/img/Spinner-1s-64px-light.gif' %}">
                        {% endif %}
                    </div>
                    <div id="rattingChart-Month"></div>
                </div>
            </div>
            <div class="col-md-6 g-2 d-none" id="rattingBarContainer-Month">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Ratting 30 Days</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="bar-loading-Month" class="d-flex justify-content-center align-items-center">
                        {% if NIGHT_MODE %}
                        <img src="{% static 'killstats/img/Spinner-1s-64px-dark.gif' %}">
                        {% else %}
                            <img src="{% static 'killstats/img/Spinner-1s-64px-light.gif' %}">
                        {% endif %}
                    </div>
                    <div id="rattingBar-Month"></div>
                </div>
            </div>
        </div>
    </div>
</div>
