noData = [
    {
        "ratting": [],
        "total": {
            "total_amount": 0,
            "total_amount_ess": 0,
            "total_amount_all": 0,
            "total_amount_mining": 0,
            "total_amount_others": 0,
            "total_amount_costs": 0,
        },
    }
]

CorpmonthlyMarch = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "alt_names": [],
                "total_amount": "200000.00",
                "total_amount_ess": "200000.00",
            }
        ],
        "total": {
            "total_amount": "200000.00",
            "total_amount_ess": "200000.00",
            "total_amount_all": "400000.00",
            "total_amount_mining": 0,
            "total_amount_others": 0,
            "total_amount_costs": 0,
        },
    }
]


Corpyearly = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "alt_names": [],
                "total_amount": "200000.00",
                "total_amount_ess": "200000.00",
            }
        ],
        "total": {
            "total_amount": "200000.00",
            "total_amount_ess": "200000.00",
            "total_amount_all": "400000.00",
            "total_amount_mining": 0,
            "total_amount_others": 0,
            "total_amount_costs": 0,
        },
    }
]

Corpdatamany = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "alt_names": [],
                "total_amount": "200000.00",
                "total_amount_ess": "200000.00",
            },
            {
                "main_id": 1010,
                "main_name": "Test1",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1019,
                "main_name": "Test10",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1020,
                "main_name": "Test11",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1021,
                "main_name": "Test12",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1011,
                "main_name": "Test2",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1012,
                "main_name": "Test3",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1013,
                "main_name": "Test4",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1014,
                "main_name": "Test5",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1015,
                "main_name": "Test6",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1016,
                "main_name": "Test7",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1017,
                "main_name": "Test8",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
            {
                "main_id": 1018,
                "main_name": "Test9",
                "alt_names": [],
                "total_amount": "100000.00",
                "total_amount_ess": "0.00",
            },
        ],
        "total": {
            "total_amount": "1400000.00",
            "total_amount_ess": "200000.00",
            "total_amount_all": "1600000.00",
            "total_amount_mining": 0,
            "total_amount_others": 0,
            "total_amount_costs": 0,
        },
    }
]
