{% load i18n %}
{% load humanize %}
{% load static %}
<table class="table table-striped table-hover w-100" id="ratting-{{ mode }}">
    <br>
    <thead>
        <th class="col-main-character w-auto">Main Character</th>
        <th class="col-total-amount w-auto">TAX Amount</th>
        <th class="col-total-ess w-auto">ESS Amount</th>
        <th class="col-total-action w-auto"></th>
    </thead>

    <tbody>
    </tbody>

    <tfoot id="foot-{{ mode }}" style="display: none;">
        <tr>
            <th><strong>{% translate "Summary" %}</strong></th>
            <th class="col-total-amount"></th>
            <th class="col-total-ess"></th>
            <th></th>
        </tr>
        <tr>
            <th><strong>{% translate "Total" %}</strong></th>
            <th class="col-total-gesamt"></th>
            <th></th>
            <th class="col-total-button"></th>
        </tr>
    </tfoot>
</table>
