{% extends 'allianceauth/base-bs5.html' %}
{% load i18n %}
{% load ledger %}
{% block main_css %}
{% endblock main_css %}
{% block extra_css %}
{% endblock extra_css %}

{% block page_title %}Ledger{% endblock %}

{% block header_nav_brand %}
    <style>
    .ledger-link {
        color: white !important; /* Stellt sicher, dass diese Farbregel Vorrang hat */
        text-decoration: none !important; /* Entfernt die Unterstreichung */
        transition: color 0.3s; /* Fügt eine sanfte Übergangsanimation hinzu */
    }

    .ledger-link:hover {
        color: rgba(255, 255, 255, 0.4) !important; /* Reduziert die Farbintensität um 60% beim Hover */
    }
    </style>
    <a class="ledger-link" href="{% url 'ledger:index' %}">Ledger</a>
{% endblock header_nav_brand %}

{% block header_nav_collapse_left %}

{% include 'ledger/partials/menu/navigation.html' %}

{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}

{% include 'ledger/partials/menu/administration.html' %}

{% endblock header_nav_collapse_right %}

{% block content %}

{% include 'ledger/bundles/ledger-css.html' %}
{% include 'bundles/datatables-css-bs5.html' %}

<div class="allianceauth-ledger-plugin card">
    <div class="container-fluid" id="vow_block">
    {% block vow_block %}{% endblock %}
    </div>
    <br>
</div>

{% endblock %}

{% block extra_javascript %}
{% endblock %}

{% block extra_script %}
{% endblock %}
