<style>
@media all {
    #ratting-Month, #ratting-Year {
        table-layout: fixed;
        word-wrap: break-word;
    }

    #ratting-Month th, #ratting-Year th,
    #ratting-Month td, #ratting-Year td {
        padding: 0.75rem;
        vertical-align: top;
    }

    #ratting-Month thead th, #ratting-Year thead th {
        text-align: center;
    }

    #ratting-Month thead th.col-main-character,
    #ratting-Year thead th.col-main-character {
        text-align: left;
    }

    #ratting-Month tbody td, #ratting-Year tbody td {
        text-align: right;
    }

    #ratting-Month tbody td:nth-child(1), #ratting-Year tbody td:nth-child(1) {
        text-align: left;
    }

    #ratting-Month tfoot th, #ratting-Year tfoot th {
        text-align: right;
    }

    #ratting-Month tfoot th:nth-child(1), #ratting-Year tfoot th:nth-child(1) {
        text-align: left;
    }

    #ratting-Month tfoot th, #ratting-Year tfoot th {
        border-top: 1px solid #dee2e6;
    }
}
</style>
