/*! * Copyright (c) 2017 ~ present NAVER Corp. * billboard.js project is licensed under the MIT license * * billboard.js, JavaScript chart library * https://naver.github.io/billboard.js/ * * @version 3.5.1 */
.bb-color-pattern{
    background-image:url(#00c73c;
    #fa7171;
    #2ad0ff;
    #7294ce;
    #e3e448;
    #cc7e6e;
    #fb6ccf;
    #c98dff;
    #4aea99;
    #bbbbbb;
    )
}
.bb svg{
    font-family:sans-serif,Arial,nanumgothic,Dotum;
    font-size:12px;
    line-height:1
}
.bb line,.bb path{
    fill:none;
    stroke:#c4c4c4
}
.bb .bb-button,.bb text{
    fill:white;
    font-size:11px;
    -webkit-user-select:none;
    -moz-user-select:none;
    -ms-user-select:none;
    user-select:none
}
.bb-axis,.bb-bars path,.bb-event-rect,.bb-legend-item-title,.bb-xgrid-focus,.bb-ygrid,.bb-ygrid-focus{
    shape-rendering:crispEdges
}
.bb-axis-y text,.bb-axis-y2 text{
    fill:white
}
.bb-event-rects{
    fill-opacity:1!important
}
.bb-event-rects .bb-event-rect{
    fill:transparent
}
.bb-event-rects .bb-event-rect._active_{
    fill:rgba(39,201,3,.05)
}
.tick._active_ text{
    fill:#00c83c!important
}
.bb-grid{
    pointer-events:none
}
.bb-grid line{
    stroke:#f1f1f1
}
.bb-xgrid-focus line,.bb-ygrid-focus line{
    stroke:#ddd
}
.bb-text.bb-empty{
    fill:#767676
}
.bb-line{
    stroke-width:1px
}
.bb-circle._expanded_{
    fill:#fff!important;
    stroke-width:2px;
    stroke:red
}
rect.bb-circle._expanded_,use.bb-circle._expanded_{
    stroke-width:1px
}
.bb-selected-circle{
    fill:#fff;
    stroke-width:2px
}
.bb-bar{
    stroke-width:0
}
.bb-bar._expanded_{
    fill-opacity:.75
}
.bb-candlestick{
    stroke-width:1px
}
.bb-candlestick._expanded_{
    fill-opacity:.75
}
.bb-circles.bb-focused,.bb-target.bb-focused{
    opacity:1
}
.bb-circles.bb-focused path.bb-line,.bb-circles.bb-focused path.bb-step,.bb-target.bb-focused path.bb-line,.bb-target.bb-focused path.bb-step{
    stroke-width:2px
}
.bb-circles.bb-defocused,.bb-target.bb-defocused{
    opacity:.3!important
}
.bb-circles.bb-defocused .text-overlapping,.bb-target.bb-defocused .text-overlapping{
    opacity:.05!important
}
.bb-region{
    fill:#4682b4;
    fill-opacity:.1
}
.bb-region.selected rect{
    fill:#27c903
}
.bb-brush .extent,.bb-zoom-brush{
    fill-opacity:.1
}
.bb-legend-item{
    user-select:none
}
.bb-legend-item-hidden{
    opacity:.15
}
.bb-legend-background{
    fill:#fff;
    stroke:#d3d3d3;
    stroke-width:1;
    opacity:.75
}
.bb-title{
    font-size:14px
}
.bb-tooltip-container{
    font-family:sans-serif,Arial,nanumgothic,Dotum;
    user-select:none;
    z-index:10
}
.bb-tooltip{
    color: black !important;
    background-color:#fff;
    border:1px solid #999;
    border-collapse:separate;
    border-spacing:0;
    empty-cells:show;
    font-size:11px;
    text-align:left
}
.bb-tooltip th{
    border-bottom:1px solid #eee;
    font-size:12px;
    padding:4px 8px;
    text-align:left
}
.bb-tooltip td{
    background-color:#fff;
    padding:4px 6px
}
.bb-tooltip td:first-child{
    padding-left:8px
}
.bb-tooltip td:last-child{
    padding-right:8px
}
.bb-tooltip td>span,.bb-tooltip td>svg{
    border-radius:5px;
    display:inline-block;
    height:10px;
    margin-right:6px;
    vertical-align:middle;
    width:10px
}
.bb-tooltip td.value{
    border-left:1px solid transparent
}
.bb-tooltip .bb-tooltip-title{
    color:#aaa;
    display:inline-block;
    line-height:20px
}
.bb-tooltip .bb-tooltip-detail table{
    border-collapse:collapse;
    border-spacing:0
}
.bb-tooltip .bb-tooltip-detail .bb-tooltip-name,.bb-tooltip .bb-tooltip-detail .bb-tooltip-value{
    color:#444;
    font-size:11px;
    font-weight:400;
    line-height:13px;
    padding:4px 0 3px;
    text-align:left
}
.bb-tooltip .bb-tooltip-detail .bb-tooltip-value{
    font-size:12px;
    font-weight:800;
    padding-left:5px
}
.bb-area{
    stroke-width:0;
    opacity:.2
}
.bb-chart-arcs-title{
    dominant-baseline:middle;
    font-size:1.3em
}
text.bb-chart-arcs-gauge-title{
    dominant-baseline:middle;
    font-size:2.7em
}
.bb-chart-arcs .bb-chart-arcs-background{
    fill:#e0e0e0;
    stroke:none
}
.bb-chart-arcs .bb-chart-arcs-gauge-unit{
    fill:#000;
    font-size:16px
}
.bb-chart-arcs .bb-chart-arcs-gauge-max,.bb-chart-arcs .bb-chart-arcs-gauge-min{
    fill:#777
}
.bb-chart-arcs .bb-chart-arcs-title{
    fill:white;
    font-size:13px!important;
    font-weight:600
}
.bb-chart-arcs path.empty{
    fill:#eaeaea;
    stroke-width:0
}
.bb-chart-arcs .bb-levels circle{
    fill:none;
    stroke:#848282;
    stroke-width:.5px
}
.bb-chart-arcs .bb-levels text{
    fill:#848282
}
.bb-chart-arc .bb-gauge-value{
    fill:#000
}
.bb-chart-arc path{
    stroke:#fff
}
.bb-chart-arc rect{
    stroke:#fff;
    stroke-width:1
}
.bb-chart-arc text{
    fill:#fff;
    font-size:13px
}
.bb-chart-radars .bb-levels polygon{
    fill:none;
    stroke:#848282;
    stroke-width:.5px
}
.bb-chart-radars .bb-levels text{
    fill:#848282
}
.bb-chart-radars .bb-axis line{
    stroke:#848282;
    stroke-width:.5px
}
.bb-chart-radars .bb-axis text{
    cursor:default;
    font-size:1.15em
}
.bb-chart-radars .bb-shapes polygon{
    fill-opacity:.2;
    stroke-width:1px
}
.bb-button{
    position:absolute;
    right:10px;
    top:10px
}
.bb-button .bb-zoom-reset{
    background-color:#fff;
    border:1px solid #ccc;
    border-radius:5px;
    cursor:pointer;
    padding:5px
}
