{% extends 'ledger/base.html' %}
{% load i18n %}
{% load humanize %}

{% block page_title %}Delete Event{% endblock %}
{% block vow_menu %}{% include 'ledger/events/menu.html' %}{% endblock %}

{% block vow_block %}
<div class="container">
    <div class="row justify-content-center">
        <div class="card-body card-primary">
            <div class="card-header">
                <h3><div class="card-title">{% translate "Delete Event" %}</div></h3>
            </div>
            <div class="card-body bg-secondary">
                <p>Möchten Sie das Event "{{ event.title }}" wirklich löschen?</p>

                <form method="post">
                    {% csrf_token %}
                    <button type="submit" class="btn btn-danger">{% translate "Ja, löschen" %}</button>
                    <a href="{% url 'ledger:event_admin' %}" class="btn btn-secondary">{% translate "Nein, abbrechen" %}</a>
                </form>
            </div>
        </div>
    </div>
</div>
{% endblock %}

{% block extra_javascript %}
{% endblock %}

{% block extra_css %}
    {% include "ledger/bundles/ledger-css.html" %}
{% endblock %}

{% block extra_script %}
{% endblock %}
